import factory
from factory import fuzzy

from django.contrib.auth.models import User

from issue_tracker import models


class UserFactory(factory.django.DjangoModelFactory):

    username = factory.Sequence(lambda n: 'john.doe{0}'.format(n))
    first_name = factory.Sequence(lambda n: 'John{0}'.format(n))
    last_name = factory.Sequence(lambda n: 'Doe{0}'.format(n))
    email = factory.Sequence(lambda n: 'joh_doe_{0}@example.com'.format(n).lower())

    class Meta:
        model = User


class IssueFactory(factory.django.DjangoModelFactory):

    name = factory.fuzzy.FuzzyText(length=100)
    created_by = factory.SubFactory('issue_tracker.tests.factories.UserFactory')
    leader = factory.SubFactory('issue_tracker.tests.factories.UserFactory')

    class Meta:
        model = models.Issue


from django import forms
from django.core.exceptions import ValidationError
from django.db.models.base import Model
from django.forms.fields import FileField
from django.forms.forms import BoundField

from .fields import *
from .models import *
from .forms import *



from pyston.filters.default_filters import NONE_LABEL


class FilterChoiceIterator:
    """
    Filter iterator surrounds default model iterator, removes blank values and adds possibilities
    to remove filtered value (blank) and filter according to "None" value.
    """

    def __init__(self, choices, field=None):
        self.choices = choices
        self.field = field

    def __iter__(self):
        yield ('', '')

        if self.field and (self.field.null or self.field.blank):
            yield ('__none__', NONE_LABEL)

        for k, v in self.choices:
            if k is not None and k != '':
                yield (k, v)

    def __len__(self):
        return len(self.choices)

    def __getattr__(self, name):
        return getattr(self.choices, name)


class UIFilterMixin:
    """
    Special mixin for improve Pyston filters. There can be defined UI appearance (widget) and used operator.
    """

    widget = None

    def get_widget(self, request):
        """
        Returns concrete widget that will be used for rendering table filter.
        """
        widget = self.widget
        if isinstance(widget, type):
            widget = widget()
        return widget

    def get_operator(self, widget):
        """
        Returns operator used for filtering. By default it is first operator defined in Pyston filter.
        """
        return self.get_allowed_operators()[0]


from django.utils.translation import ugettext_lazy as _

from is_core.exceptions.response import response_exception_factory


def csrf_failure(request, reason=''):
    return response_exception_factory(request, 403, _('Csrf Token expired'), reason)
